#ifndef GLBACKGROUND_H
#define GLBACKGROUND_H

/*
  Copyright (C) 2001-2002 Leonardo Boselli

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  Please send remarks, questions and bug reports to
    boselli@uno.it
  or write to:
    Leonardo Boselli
    Via Paoletti, 33
    18100 Imperia
    ITALY
*/

#include "glcolor.h"
#include "gltexture.h"

//
// GLBackground
//
class GLBackground: public M3Reference {
private:
  GLColor color;
  int srcBlend;
  int dstBlend;
public:
  GLBackground(
    float r = 1, float g = 1, float b = 1, float a = 1,
    int src = GL_ONE, int dst = GL_ZERO
  );
  virtual ~GLBackground();
  void setColor(float r, float g, float b) {color.set(r,g,b);}
  void setColor(float r, float g, float b, float a) {color.set(r,g,b,a);}
  void setBlendFactors(int src, int dst) {srcBlend = src; dstBlend = dst;}
  int getSrcBlend() {return srcBlend;}
  int getDstBlend() {return dstBlend;}
  GLColor& getColor() {return color;}
  virtual void render(bool blend = false) = 0;
};

//
// GLStarfield
//
class GLStarfield: public GLBackground {
private:
  unsigned int list;
  GLTexture* texture;
private:
  void generateStars(float rr, int num);
public:
  GLStarfield(float rr, int num, GLTexture* txt, float sz);
  ~GLStarfield();
  void render(bool blend = false);
};

//
// GLSkybox
//
class GLSkybox: public GLBackground {
protected:
  unsigned int list;
  int texturesCount;
  GLTexture** textures;
private:
  void init(int txtCount, GLTexture** txts);
protected:
  GLSkybox(int txtCount, GLTexture** txts, int src = GL_ONE, int dst = GL_ONE);
public:
  GLSkybox(GLTexture** txts);
  virtual ~GLSkybox();
  void render(bool blend = false);
};

//
// GLMirroredSkybox
//
class GLMirroredSkybox: public GLSkybox {
public:
  GLMirroredSkybox(GLTexture** txts, int src = GL_ONE, int dst = GL_ONE);
};

#endif // GLBACKGROUND_H
