/*
  Copyright (C) 2001-2002 Leonardo Boselli

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  Please send remarks, questions and bug reports to
    boselli@uno.it
  or write to:
    Leonardo Boselli
    Via Paoletti, 33
    18100 Imperia
    ITALY
*/

#include "glcamera.h"

#include "opengl.h"
#include "glwin.h"
#include "fmsound.h"

//
// GLFrustum
//
GLFrustum::GLFrustum(): clipPlaneNear(0), clipPlaneFar(0) {
}

void GLFrustum::update() {
  float prj[16];
  float mdl[16];
  float clp[16];
  float invT;
  glGetFloatv(GL_PROJECTION_MATRIX,prj);
  glGetFloatv(GL_MODELVIEW_MATRIX,mdl);
  clp[ 0] = mdl[ 0]*prj[ 0]+mdl[ 1]*prj[ 4]+mdl[ 2]*prj[ 8]+mdl[ 3]*prj[12];
  clp[ 1] = mdl[ 0]*prj[ 1]+mdl[ 1]*prj[ 5]+mdl[ 2]*prj[ 9]+mdl[ 3]*prj[13];
  clp[ 2] = mdl[ 0]*prj[ 2]+mdl[ 1]*prj[ 6]+mdl[ 2]*prj[10]+mdl[ 3]*prj[14];
  clp[ 3] = mdl[ 0]*prj[ 3]+mdl[ 1]*prj[ 7]+mdl[ 2]*prj[11]+mdl[ 3]*prj[15];
  clp[ 4] = mdl[ 4]*prj[ 0]+mdl[ 5]*prj[ 4]+mdl[ 6]*prj[ 8]+mdl[ 7]*prj[12];
  clp[ 5] = mdl[ 4]*prj[ 1]+mdl[ 5]*prj[ 5]+mdl[ 6]*prj[ 9]+mdl[ 7]*prj[13];
  clp[ 6] = mdl[ 4]*prj[ 2]+mdl[ 5]*prj[ 6]+mdl[ 6]*prj[10]+mdl[ 7]*prj[14];
  clp[ 7] = mdl[ 4]*prj[ 3]+mdl[ 5]*prj[ 7]+mdl[ 6]*prj[11]+mdl[ 7]*prj[15];
  clp[ 8] = mdl[ 8]*prj[ 0]+mdl[ 9]*prj[ 4]+mdl[10]*prj[ 8]+mdl[11]*prj[12];
  clp[ 9] = mdl[ 8]*prj[ 1]+mdl[ 9]*prj[ 5]+mdl[10]*prj[ 9]+mdl[11]*prj[13];
  clp[10] = mdl[ 8]*prj[ 2]+mdl[ 9]*prj[ 6]+mdl[10]*prj[10]+mdl[11]*prj[14];
  clp[11] = mdl[ 8]*prj[ 3]+mdl[ 9]*prj[ 7]+mdl[10]*prj[11]+mdl[11]*prj[15];
  clp[12] = mdl[12]*prj[ 0]+mdl[13]*prj[ 4]+mdl[14]*prj[ 8]+mdl[15]*prj[12];
  clp[13] = mdl[12]*prj[ 1]+mdl[13]*prj[ 5]+mdl[14]*prj[ 9]+mdl[15]*prj[13];
  clp[14] = mdl[12]*prj[ 2]+mdl[13]*prj[ 6]+mdl[14]*prj[10]+mdl[15]*prj[14];
  clp[15] = mdl[12]*prj[ 3]+mdl[13]*prj[ 7]+mdl[14]*prj[11]+mdl[15]*prj[15];
  frustum[0][0] = clp[ 3]-clp[ 0];
  frustum[0][1] = clp[ 7]-clp[ 4];
  frustum[0][2] = clp[11]-clp[ 8];
  frustum[0][3] = clp[15]-clp[12];
  invT = 1/sqrt(
    frustum[0][0]*frustum[0][0]+
    frustum[0][1]*frustum[0][1]+
    frustum[0][2]*frustum[0][2]
  );
  frustum[0][0] *= invT;
  frustum[0][1] *= invT;
  frustum[0][2] *= invT;
  frustum[0][3] *= invT;
  frustum[1][0] = clp[ 3]+clp[ 0];
  frustum[1][1] = clp[ 7]+clp[ 4];
  frustum[1][2] = clp[11]+clp[ 8];
  frustum[1][3] = clp[15]+clp[12];
  invT = 1/sqrt(
    frustum[1][0]*frustum[1][0]+
    frustum[1][1]*frustum[1][1]+
    frustum[1][2]*frustum[1][2]
  );
  frustum[1][0] *= invT;
  frustum[1][1] *= invT;
  frustum[1][2] *= invT;
  frustum[1][3] *= invT;
  frustum[2][0] = clp[ 3]+clp[ 1];
  frustum[2][1] = clp[ 7]+clp[ 5];
  frustum[2][2] = clp[11]+clp[ 9];
  frustum[2][3] = clp[15]+clp[13];
  invT = 1/sqrt(
    frustum[2][0]*frustum[2][0]+
    frustum[2][1]*frustum[2][1]+
    frustum[2][2]*frustum[2][2]
  );
  frustum[2][0] *= invT;
  frustum[2][1] *= invT;
  frustum[2][2] *= invT;
  frustum[2][3] *= invT;
  frustum[3][0] = clp[ 3]-clp[ 1];
  frustum[3][1] = clp[ 7]-clp[ 5];
  frustum[3][2] = clp[11]-clp[ 9];
  frustum[3][3] = clp[15]-clp[13];
  invT = 1/sqrt(
    frustum[3][0]*frustum[3][0]+
    frustum[3][1]*frustum[3][1]+
    frustum[3][2]*frustum[3][2]
  );
  frustum[3][0] *= invT;
  frustum[3][1] *= invT;
  frustum[3][2] *= invT;
  frustum[3][3] *= invT;
  frustum[4][0] = clp[ 3]-clp[ 2];
  frustum[4][1] = clp[ 7]-clp[ 6];
  frustum[4][2] = clp[11]-clp[10];
  frustum[4][3] = clp[15]-clp[14];
  invT = 1/sqrt(
    frustum[4][0]*frustum[4][0]+
    frustum[4][1]*frustum[4][1]+
    frustum[4][2]*frustum[4][2]
  );
  frustum[4][0] *= invT;
  frustum[4][1] *= invT;
  frustum[4][2] *= invT;
  frustum[4][3] *= invT;
  frustum[5][0] = clp[ 3]+clp[ 2];
  frustum[5][1] = clp[ 7]+clp[ 6];
  frustum[5][2] = clp[11]+clp[10];
  frustum[5][3] = clp[15]+clp[14];
  invT = 1/sqrt(
    frustum[5][0]*frustum[5][0]+
    frustum[5][1]*frustum[5][1]+
    frustum[5][2]*frustum[5][2]
  );
  frustum[5][0] *= invT;
  frustum[5][1] *= invT;
  frustum[5][2] *= invT;
  frustum[5][3] *= invT;
}

//
// GLCamera
//

GLCamera::GLCamera(): fieldOfView(60), renderingReflections(false) {
}

void GLCamera::setOrtho() {
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluOrtho2D(-1,1,-1,1);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}

void GLCamera::saveView() {
  glMatrixMode(GL_PROJECTION);
  glPushMatrix();
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
}

void GLCamera::restoreView() {
  glMatrixMode(GL_MODELVIEW);
  glPopMatrix();
  glMatrixMode(GL_PROJECTION);
  glPopMatrix();
  glMatrixMode(GL_MODELVIEW);
}

void GLCamera::setPerspective(float fov, float aspect, float zn, float zf) {
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(
    fieldOfView = fov, aspect, clipPlaneNear = zn, clipPlaneFar = zf
  );
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}

void GLCamera::apply() {
  M3Vector loc = getPosition();
  M3Vector lookAt(loc);
  lookAt.add(getViewDirectionX(),getViewDirectionY(),getViewDirectionZ());
  glLoadIdentity();
  gluLookAt(
    loc.getX(),loc.getY(),loc.getZ(),
    lookAt.getX(),lookAt.getY(),lookAt.getZ(),
    getUpDirectionX(),getUpDirectionY(),getUpDirectionZ()
  );
  GLFrustum::update();
  float invNorm = 1.0f/sqrt(
    getViewDirectionX()*getViewDirectionX()+
    getViewDirectionZ()*getViewDirectionZ()
  );
  if(invNorm == 0) invNorm = 1;
  viewXY.set(invNorm*getViewDirectionX(),0,invNorm*getViewDirectionZ());
  M3Vector velocity(getPositionX(),getPositionY(),-getPositionZ());
  velocity.subtract(lastApplyPosition).scale(GLWin::getInvTimeStep());
  lastApplyPosition.set(getPositionX(),getPositionY(),-getPositionZ());
  FMListener::setAttributes(
    getViewDirectionX(),getViewDirectionY(),-getViewDirectionZ(),
    getUpDirectionX(),getUpDirectionY(),-getUpDirectionZ(),
    lastApplyPosition.getFloatArray(),velocity.getFloatArray()
  );
}

void GLCamera::applyRot() {
  M3Vector dir = getViewDirection();
  M3Vector up = getUpDirection();
  glLoadIdentity();
  gluLookAt(
    0,0,0, dir.getX(),dir.getY(),dir.getZ(), up.getX(),up.getY(),up.getZ()
  );
}
