#ifndef GLPARTICLE_H
#define GLPARTICLE_H

/*
  Copyright (C) 2001-2002 Leonardo Boselli

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  Please send remarks, questions and bug reports to
    boselli@uno.it
  or write to:
    Leonardo Boselli
    Via Paoletti, 33
    18100 Imperia
    ITALY
*/

#include "globject.h"
#include "gltexture.h"
#include "math3d.h"
#include "glcolor.h"

class GLParticle;
class GLEmitter;

//
// GLParticle
//
class GLParticle {
private:
  float life;
  M3Vector startPosition;
  M3Vector startVelocity;
public:
  GLParticle();
  void setLife(float lf) {life = lf;}
  float getLife() {return life;}
  float incrementLife(float step) {return life += step;}
  void setStartPosition(M3Vector& pos) {startPosition = pos;}
  M3Vector& getStartPosition() {return startPosition;}
  void setStartVelocity(M3Vector& vel) {startVelocity = vel;}
  M3Vector& getStartVelocity() {return startVelocity;}
  void render(M3Vector& pos, M3Vector& p1, M3Vector& p2, GLColor& color);
};

//
// GLEmitter
//
class GLEmitter: public GLObject {
private:
  int particlesCount;
  GLParticle** particles;
  GLTexture* particleTexture;
  bool hasAlpha;
  M3Vector* emitterVelocity;
  M3Vector particleVelocity;
  float speedVar;
  float startSize, stepSize;
  GLColor startColor, stepColor;
  M3Vector startGravity, stepGravity;
  float maxLife;
  bool oneShot;
protected:
  float lastRenderingTime;
  M3Vector lastRenderingPosition;
  void resetParticle(int idx, float lf, float invRenderingStep);
public:
  GLEmitter(
    int pCount, float mxLif, float maxRadius, bool updateVelocity = false
  );
  ~GLEmitter();
  void reset();
  void setParticleTexture(GLTexture* txt, bool ha = false) {
    hasAlpha = ha;
    if(particleTexture) particleTexture->releaseReference();
    particleTexture = txt->acquireReference();
  }
  void setParticleVelocity(M3Vector& vel, float speVar)
    {particleVelocity.set(vel); speedVar = speVar;}
  void setColor(GLColor& start, GLColor& end) {
    startColor.set(start);
    stepColor.set(end).subtract(start).scale(1/maxLife);
  }
  void setSize(float start, float end)
    {startSize = start; stepSize = (end-start)/maxLife;}
  void setGravity(M3Vector& start, M3Vector& end) {
    startGravity.set(start);
    stepGravity.set(end).subtract(start).scale(1/maxLife);
  }
  void setMaxLife(float ml) {maxLife = ml;}
  bool getMaxLife() {return maxLife;}
  void setOneShot(bool os) {oneShot = os;}
  bool isOneShot() {return oneShot;}
  void render(GLCamera& camera);
};

#endif // GLPARTICLE_H
