#ifndef GLRENDERER_H
#define GLRENDERER_H

/*
  Copyright (C) 2001-2002 Leonardo Boselli

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  Please send remarks, questions and bug reports to
    boselli@uno.it
  or write to:
    Leonardo Boselli
    Via Paoletti, 33
    18100 Imperia
    ITALY
*/

#include "glworld.h"
#include "glfont.h"

//
// GLRenderer
//
class GLRenderer {
private:
  bool vSyncOn;
  int width;
  int height;
  int colorDepth;
  int depthDepth;
  int stencilDepth;
protected:
  void setVSyncOn(bool b) {vSyncOn = b;}
  void applyVSync();
  void setWidth(int w) {width = w;}
  void setHeight(int h) {height = h;}
  void setColorDepth(int cd) {colorDepth = cd;}
  void setDepthDepth(int dd) {depthDepth = dd;}
  void setStencilDepth(int sd) {stencilDepth = sd;}
  void setSize(int w, int h) {setWidth(w); setHeight(h);}
public:
  bool isVSyncOn() {return vSyncOn;}
  int getWidth() {return width;}
  int getHeight() {return height;}
  int getColorDepth() {return colorDepth;}
  int getDepthDepth() {return depthDepth;}
  int getStencilDepth() {return stencilDepth;}
private:
  GLWorld* world;
  GLCamera* camera;
public:
  GLRenderer();
  ~GLRenderer();
  bool initializeRenderer();
  void finalizeRenderer();
  void executeRendering();
  virtual void drawTextOverlay() {};
  virtual GLFont* getTextOverlayFont() {return NULL;}
  virtual bool isTextOverlayDrawn() {return false;}
  GLCamera* setCamera(GLCamera* newCamera)
    {GLCamera* oldCamera = camera; camera = newCamera; return oldCamera;}
  GLCamera& getCamera() {return *camera;}
  void setPerspective(float fov, float zn, float zf) {
    camera->setPerspective(fov,(float)width/(float)height,zn,zf);
  }
  void updatePerspective()
    {if(camera) camera->updatePerspective((float)width/(float)height);}
  GLWorld& getWorld() {return *world;}
};

#endif // GLRENDERER_H
