#ifndef GLTEXTURE_H
#define GLTEXTURE_H

/*
  Copyright (C) 2001-2002 Leonardo Boselli

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  Please send remarks, questions and bug reports to
    boselli@uno.it
  or write to:
    Leonardo Boselli
    Via Paoletti, 33
    18100 Imperia
    ITALY
*/

#include "opengl.h"
#include "dataread.h"
#include "reference.h"

class DRImage;

//
// GLTexture
//
class GLTexture: public Reference {
private:
  unsigned int textureID;
protected:
  GLTexture(unsigned int id);
  virtual void destroyObject();
  unsigned int getTextureID() {return textureID;}
public:
  GLTexture(DRImage& img, bool repeat = false);
  virtual GLTexture* acquireReference()
    {return dynamic_cast<GLTexture*>(Reference::acquireReference());}
  virtual void apply() {glBindTexture(GL_TEXTURE_2D,textureID);}
};

//
// GLAnimatedTexture
//
class GLAnimatedTexture: public GLTexture {
private:
  unsigned int* textureIDs;
  float loopDuration;
  bool animated;
protected:
  void destroyObject();
public:
  GLAnimatedTexture(
    int imgsCount, DRImage** imgs, float duration, bool repeat = false
  );
  GLTexture* acquireReference()
    {return dynamic_cast<GLAnimatedTexture*>(Reference::acquireReference());}
  unsigned int getTexturesCount() {return getTextureID();}
  void setLoopDuration(float ld) {loopDuration = ld;}
  float getLoopDuration() {return loopDuration;}
  bool isAnimated() {return animated;}
  void setAnimated(bool a) {animated = a;}
  void apply(int idx) {glBindTexture(GL_TEXTURE_2D,textureIDs[idx]);}
  void apply();
};

//
// GLBumpedTexture
//
class GLBumpedTexture: public Reference {
private:
  unsigned int textureID[2];
protected:
  void destroyObject();
public:
  GLBumpedTexture(DRImage& img, bool repeat = false);
  GLBumpedTexture* acquireReference()
    {return dynamic_cast<GLBumpedTexture*>(Reference::acquireReference());}
  void applyDirect()
    {glBindTexture(GL_TEXTURE_2D,textureID[0]);}
  void applyInverse()
    {glBindTexture(GL_TEXTURE_2D,textureID[1]);}
};

#endif // GLTEXTURE_H
