/*
  Copyright (C) 2001-2002 Leonardo Boselli

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  Please send remarks, questions and bug reports to
    boselli@uno.it
  or write to:
    Leonardo Boselli
    Via Paoletti, 33
    18100 Imperia
    ITALY
*/

#include "initdlg.h"
#include "glwin.h"

#include <stdlib.h> // for itoa()

//
// InitDlgProc
//
BOOL CALLBACK InitDlgProc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp) {
  static GLWin* app = NULL;
  static char initModeStr[16] = "\0";
  static bool initFullscreen = false;
  static bool initVSync = false;
  static int initDepthBits = 16;
  static int initStencilBits = 0;
  switch(msg) {
    case WM_INITDIALOG: {
      app = (GLWin*)lp;
      bool initFile = false;
      HANDLE file = CreateFile(
        "setup.ini",GENERIC_READ,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL
      );
      if(file != INVALID_HANDLE_VALUE) {
        initFile = true;
        char buffer[64];
        DWORD bytesRead;
      	ReadFile(file,buffer,64,&bytesRead,NULL);
        CloseHandle(file);
        char *token = strtok(buffer,",");
        strncpy(initModeStr,token,15);
        initModeStr[15] = '\0';
        token = strtok(NULL,",");
        initFullscreen = token[0] == 'F' || token[0] == 'f';
        token = strtok(NULL,",");
        initVSync = token[0] == 'V' || token[0] == 'v';
        token = strtok(NULL,",");
        initDepthBits = atoi(token);
        token = strtok(NULL,",");
        initStencilBits = atoi(token);
      }
      SendDlgItemMessage(
        hwnd,IDC_Logo,STM_SETIMAGE,IMAGE_BITMAP,
        (LPARAM)LoadImage(
          GetModuleHandle(NULL),MAKEINTRESOURCE(1),IMAGE_BITMAP,0,0,
          LR_DEFAULTCOLOR|LR_CREATEDIBSECTION
        )
      );
      HWND modeList = GetDlgItem(hwnd,IDC_ScreenSize);
      DEVMODE devMode;
      int index = 0;
      while(EnumDisplaySettings(NULL,index++,&devMode)) {
        if(devMode.dmBitsPerPel >= 16) {
          char modeStr[16], valStr[8];
          strcat(strcpy(modeStr,itoa(devMode.dmPelsWidth,modeStr,10)),"x");
          strcat(strcat(modeStr,itoa(devMode.dmPelsHeight,valStr,10)),"x");
          strcat(modeStr,itoa(devMode.dmBitsPerPel,valStr,10));
          SendMessage(modeList,LB_INSERTSTRING,0,(LPARAM)modeStr);
        }
      }
      if(initFile)
        SendMessage(modeList,LB_SELECTSTRING,(WPARAM)0,(LPARAM)initModeStr);
      else
        SendMessage(modeList,LB_SETCURSEL,(WPARAM)0,(LPARAM)0);
      CheckDlgButton(
        hwnd,initFullscreen?IDC_Fullscreen:IDC_Windowed,BST_CHECKED
      );
      CheckDlgButton(hwnd,initVSync?IDC_VSyncOn:IDC_VSyncOff,BST_CHECKED);
      CheckDlgButton(hwnd,
        initDepthBits == 32? IDC_32depth:
        initDepthBits == 24? IDC_24depth:
        IDC_16depth, BST_CHECKED
      );
      CheckDlgButton(
        hwnd, initStencilBits == 0? IDC_0stencil: IDC_8stencil,BST_CHECKED
      );
      return TRUE;
    }
    case WM_COMMAND: {
      WORD id = LOWORD(wp);
      switch(id) {
        case IDOK: {
          bool valuesChanged = false;
          HWND modeList = GetDlgItem(hwnd,IDC_ScreenSize);
          LRESULT idx = SendMessage(modeList,LB_GETCURSEL,(WPARAM)0,(LPARAM)0);
          if(idx == LB_ERR) {
            EndDialog(hwnd,-1);
            return TRUE;
          }
          char modeStr[16], copyModeStr[16];
          SendMessage(modeList,LB_GETTEXT,(WPARAM)idx,(LPARAM)modeStr);
          strcpy(copyModeStr,modeStr);
          if(stricmp(copyModeStr,initModeStr) != 0) valuesChanged = true;
          int width = 640, height = 480, depth = 16;
          char *token = strtok(copyModeStr,"x");
          if(token) {
            width = atoi(token);
            token = strtok(NULL,"x");
            if(token) {
              height = atoi(token);
              token = strtok(NULL,"x");
              if(token) {
                depth = atoi(token);
              }
            }
          }
          app->setFullSize(width,height);
          app->setColorDepth(depth);
          if(IsDlgButtonChecked(hwnd,IDC_32depth) == BST_CHECKED) {
            app->setDepthDepth(32);
          } else if(IsDlgButtonChecked(hwnd,IDC_24depth) == BST_CHECKED) {
            app->setDepthDepth(24);
          } else if(IsDlgButtonChecked(hwnd,IDC_16depth) == BST_CHECKED) {
            app->setDepthDepth(16);
          } else {
            EndDialog(hwnd,-1);
            return TRUE;
          }
          if(app->getDepthDepth() != initDepthBits) valuesChanged = true;
          if(IsDlgButtonChecked(hwnd,IDC_8stencil) == BST_CHECKED) {
            app->setStencilDepth(8);
          } else if(IsDlgButtonChecked(hwnd,IDC_0stencil) == BST_CHECKED) {
            app->setStencilDepth(0);
          } else {
            EndDialog(hwnd,-1);
            return TRUE;
          }
          if(app->getStencilDepth() != initStencilBits) valuesChanged = true;
          if(IsDlgButtonChecked(hwnd,IDC_Fullscreen) == BST_CHECKED) {
            app->setFullscreen(true);
          } else if(IsDlgButtonChecked(hwnd,IDC_Windowed) == BST_CHECKED) {
            app->setFullscreen(false);
          } else {
            EndDialog(hwnd,-1);
            return TRUE;
          }
          if(app->isFullscreen() != initFullscreen) valuesChanged = true;
          if(IsDlgButtonChecked(hwnd,IDC_VSyncOn) == BST_CHECKED) {
            app->setVSyncOn(true);
          } else if(IsDlgButtonChecked(hwnd,IDC_VSyncOff) == BST_CHECKED) {
            app->setVSyncOn(false);
          } else {
            EndDialog(hwnd,-1);
            return TRUE;
          }
          if(app->isVSyncOn() != initVSync) valuesChanged = true;
          EndDialog(hwnd,IDOK);
          if(valuesChanged) {
            HANDLE file = CreateFile(
              "setup.ini",GENERIC_WRITE,0,NULL,CREATE_ALWAYS,
              FILE_ATTRIBUTE_NORMAL,NULL
            );
            if(file != INVALID_HANDLE_VALUE) {
              char buffer[64];
              char valStr[8];
              strcat(strcpy(buffer,modeStr),",");
              strcat(buffer,app->isFullscreen()?"FULLSCREEN,":"WINDOWED,");
              strcat(buffer,app->isVSyncOn()?"VSYNC,":"NO VSYNC,");
              strcat(strcat(buffer,itoa(app->getDepthDepth(),valStr,10)),",");
              strcat(buffer,itoa(app->getStencilDepth(),valStr,10));
              DWORD bytesWritten;
            	WriteFile(file,buffer,strlen(buffer),&bytesWritten,NULL);
              CloseHandle(file);
            }
          }
          return TRUE;
        }
        case IDCANCEL: {
          EndDialog(hwnd,IDCANCEL);
          return TRUE;
        }
        default:
          return FALSE;
      }
    }
    default:
      return FALSE;
  }
}
