/*
  Copyright (C) 2001-2002 Leonardo Boselli

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  Please send remarks, questions and bug reports to
    boselli@uno.it
  or write to:
    Leonardo Boselli
    Via Paoletti, 33
    18100 Imperia
    ITALY
*/

#include "slinterpreter.h"

//
// SLInterpreter
//

SLInterpreter::SLInterpreter():
  program(NULL), running(false), cpuFrequency(10000)
{}

SLInterpreter::~SLInterpreter() {
  if(program) delete program;
}

int SLInterpreter::registerNatives() {
  extern AMX_NATIVE_INFO core_Natives[];
  extern AMX_NATIVE_INFO math_Natives[];
  amx_Register(&amx,core_Natives,-1);
  return amx_Register(&amx,math_Natives,-1);
}

bool SLInterpreter::load(DRData* prog) {
  running = false;
  ZeroMemory(&amx, sizeof amx);
  if(amx_Init(&amx,prog->getBuffer()) == AMX_ERR_NONE) {
    prog->setOwnsBuffer(false);
    if(program) delete program;
    program = prog->getBuffer();
    return (registerNatives() == AMX_ERR_NONE);
  }
  return false;
}

bool SLInterpreter::load(char* fileName) {
  DRDataFile file(fileName);
  DRData* data = file.getData();
  if(data) {
    bool ret = load(data);
    delete data;
    return ret;
  }
  return false;
}